/**
 *******************************************************************************
 * @file  kernel.h
 * @brief Kernel specification (Service Calls, Data Types, Constants, Macros)
 * 
 * @note 
 * Copyright (C) 2011 Renesas Electronics Corporation 
 * Copyright (C) 2011 Renesas Micro Systems Co., Ltd. 
 * 
 * @par
 *  This is a sample program. 
 *  Renesas Electronics assumes no responsibility for any losses incurred. 
 * 
 *******************************************************************************
 */

#ifndef	KERNEL_H__
#define	KERNEL_H__

/*==========================================================================*/
/* I N C L U D E															*/
/*==========================================================================*/
#include <itron.h>							// ITRON General Definitions (Data Types, Constants, Macros)

/*===========================================================================*/
/* S T R U C T                                                               */
/*===========================================================================*/
//=============================================================
// Task management function
//=============================================================
//-------------------------------------
// Task creation information packet
//-------------------------------------
typedef	struct t_ctsk {
	ATR		tskatr;						/*!< Task attribute							*/
	VP_INT	exinf;						/*!< Task extended information				*/
	FP		task;						/*!< Task start address						*/
	PRI		itskpri;					/*!< Task initial priority					*/
	SIZE	stksz;						/*!< Task stack size						*/
	VP		stk;						/*!< Base address of task stack space		*/
} T_CTSK;

//=============================================================
// Synchronization and communication functions
//=============================================================
//-------------------------------------
// Semaphore creation infomation packet
//-------------------------------------
typedef struct t_csem {
	ATR		sematr;						/*!< Semaphore attribute					*/
	UINT	isemcnt;					/*!< Initial semaphore resource count		*/
	UINT	maxsem;						/*!< Maximum semaphore resource count		*/
}T_CSEM;

//-------------------------------------
// Eventflag creation infomation packet
//-------------------------------------
typedef struct t_cflg {
	ATR		flgatr;						/*!< Eventflag attribute					*/
	FLGPTN	iflgptn;					/*!< Initial value of eventflag bit pattern	*/
}T_CFLG;

//-------------------------------------
// Mailbox creation infomation packet
//-------------------------------------
typedef struct t_cmbx {
	ATR     mbxatr;                     /*!< Mailbox attribute						*/
	PRI		maxmpri;                    /*!< Maximum message priority				*/
	VP		mprihd;						/*!< Start address of the area for message	
										     queue headers for each message priority */
}T_CMBX;

//=============================================================
// Extended synchronization and communication functions
//=============================================================
//-------------------------------------
// Mutex creation infomation packet
//-------------------------------------
typedef struct t_cmtx {
    ATR     mtxatr;                     /*!< Mutex attribute						*/
	PRI		ceilpri;					/*!< Mutex ceiling priority					*/
}T_CMTX;

//=============================================================
// Interrupt management functions
//=============================================================
//-------------------------------------
// Interrupt service routine information packet
//-------------------------------------
typedef struct t_dinh {
	ATR		inhatr;						/*!< Interrupt handler attribute			*/
	FP		inthdr;						/*!< Interrupt handler start address		*/
} T_DINH;

/*==========================================================================*/
/* F U N C T I O N   P R O T O T Y P E										*/
/*==========================================================================*/

//-------------------------------------------------------------
// Task Management Functions
//-------------------------------------------------------------
ER sta_tsk(ID tskid, VP_INT stacd);
void ext_tsk(void);
ER ter_tsk(ID tskid);
ER chg_pri(ID tskid, PRI tskpri);
ER get_pri(ID tskid, PRI *p_tskpri);

//-------------------------------------------------------------
// Task Dependant Synchronization Functions
//-------------------------------------------------------------
ER slp_tsk(void);
ER tslp_tsk(TMO tmout);
ER wup_tsk(ID tskid);
ER iwup_tsk(ID tskid);
ER_UINT can_wup(ID tskid);
ER rel_wai(ID tskid);
ER irel_wai(ID tskid);

//-------------------------------------------------------------
// Synchronization and Communication Functions
//-------------------------------------------------------------
// ------ Semaphores ------
ER cre_sem(ID semid, T_CSEM *pk_csem);
ER del_sem(ID semid);
ER wai_sem(ID semid);
ER pol_sem(ID semid);
ER twai_sem(ID semid, TMO tmout);
ER sig_sem(ID semid);
ER isig_sem(ID semid);

// ------ Eventflags ------
ER cre_flg(ID flgid, T_CFLG *pk_cflg);
ER del_flg(ID flgid);
ER set_flg(ID flgid, FLGPTN setptn);
ER iset_flg(ID flgid, FLGPTN setptn);
ER clr_flg(ID flgid, FLGPTN clrptn);
ER wai_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn);
ER pol_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn);
ER twai_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn, TMO tmout);

// ------ Mailboxs ------
ER cre_mbx(ID mbxid, T_CMBX *pk_cmbx);
ER del_mbx(ID mbxid);
ER snd_mbx(ID mbxid, T_MSG *pk_msg);
ER rcv_mbx(ID mbxid, T_MSG **ppk_msg);
ER prcv_mbx(ID mbxid, T_MSG **ppk_msg);
ER trcv_mbx(ID mbxid, T_MSG **ppk_msg, TMO tmout);

//-------------------------------------------------------------
// Extended Synchronization and Communication Functions
//-------------------------------------------------------------
// ------ Mutexes ------
ER cre_mtx(ID mtxid, T_CMTX *pk_cmtx);
ER del_mtx(ID mtxid);
ER loc_mtx(ID mtxid);
ER ploc_mtx(ID mtxid);
ER tloc_mtx(ID mtxid, TMO tmout);
ER unl_mtx(ID mtxid);

//-------------------------------------------------------------
// Time Management Functions
//-------------------------------------------------------------
ER set_tim(SYSTIM *p_systim);
ER get_tim(SYSTIM *p_systim);

//-------------------------------------------------------------
// System State Management Functions
//-------------------------------------------------------------
ER rot_rdq(PRI tskpri);
ER irot_rdq(PRI tskpri);
ER get_tid(ID *p_tskid);
ER iget_tid(ID *p_tskid);
ER loc_cpu(void);
ER unl_cpu(void);
BOOL sns_loc(void);
ER dis_dsp(void);
ER ena_dsp(void);


/*==========================================================================*/
/* D E F I N E																*/
/*==========================================================================*/
//=============================================================
// Constants
//=============================================================
//-------------------------------------
// Object Attribute
//-------------------------------------
// ------ For task ------
#define	TA_HLNG		0x00				/*!< Start a processing though a high-level language interface	*/
#define	TA_ASM		0x01				/*!< Start a processing though an assembly language interface	*/
#define	TA_ACT		0x02				/*!< Task is activated after the creation						*/

// ------ For semaphores, eventflags,,,  ------
#define	TA_TFIFO	0x00				/*!< Task wait queue is in FIFO order							*/
#define	TA_TPRI		0x01				/*!< Task wait queue is in task priority order					*/

// ------ For eventflags  ------
#define	TA_WMUL		0x02				/*!< Multiple tasks are allowed to be in the waiting state 
										     for the evntflag											*/
#define	TA_CLR		0x04				/*!< Eventflag's bit pattern is cleared when a task is released
											 from the waiting state for that eventflag					*/

// ------ For mailbox ------
#define TA_MFIFO	0x00				/*!< Message queue is in FIFO order								*/
#define TA_MPRI		0x02                /*!< Message queue is in message priority order					*/

// ------ For mutex  ------
#define TA_INHERIT	0x02				/*!< Mutex uses the priority inheriance protocol (Not used)		*/	
#define TA_CEILING  0x03                /*!< Mutex uses the priority ceiling protocol (Not used)		*/

//-------------------------------------
// Service Call Opetational Mode
//-------------------------------------
// ------ For eventflags  ------
#define	TWF_ANDW	0x00				/*!< AND waiting condition for an eventflag						*/
#define	TWF_ORW		0x01				/*!< OR  waiting condition for an eventflag						*/

//-------------------------------------
// Other constants
//-------------------------------------
// ------ For task ------
#define TSK_SELF	0x00				/*!< Specifying invoking task									*/
#define TSK_NONE	0x00				/*!< NO applicable task											*/

#define TPRI_SELF	0x00				/*!< Specifying the vase priority of the invoking task			*/
#define TPRI_INI	0x00				/*!< Specifying the initial priority of the task				*/

//=============================================================
// Kernel Configuration Constants (depend on HW-RTOS architecture)
//=============================================================
//-------------------------------------
// Priority Range
//-------------------------------------
#define TMIN_TPRI	 1					/*!< Minimum task priority(=1)									*/
#define TMAX_TPRI   15					/*!< Maximum task priority										*/

#define TMIN_MPRI    1                  /*!< Minimum message priority(=1)                               */
#define TMAX_MPRI    7                  /*!< Maximum message priority                                   */

//-------------------------------------
// Number of Bits in Bitpatterns
//-------------------------------------
#define TBIT_FLGPTN	16					/*!< Number of bits asn eventflag								*/

//-------------------------------------
// Others
//-------------------------------------
#define TMAX_MAXSEM	31					/*!< Maximum value of the maximum definable semaphore resource count */

///////////////////////////////////////////////////////////////////////////////////////////////
// HW-RTOS original
///////////////////////////////////////////////////////////////////////////////////////////////

/*===========================================================================*/
/* S T R U C T                                                               */
/*===========================================================================*/
//=============================================================
// Static table
//=============================================================
//-------------------------------------
// Static task table packet
//-------------------------------------
typedef struct task_table {
	ID	id;								/*!< Task ID								*/
	T_CTSK	t_ctsk;						/*!< Task creation information packet		*/
}TSK_TBL;

//-------------------------------------
// Static semaphore table packet
//-------------------------------------
typedef struct semaphore_table {
	ID	id;								/*!< Semaphore ID							*/
	T_CSEM	pk_ccem;					/*!< Semaphore creation infomation packet	*/
}SEM_TBL;

//-------------------------------------
// Static eventflag table packet
//-------------------------------------
typedef struct flag_table {
	ID	id;								/*!< Eventflag ID							*/
	T_CFLG	pk_cflg;					/*!< Eventflag creation infomation packet	*/
}FLG_TBL;

//-------------------------------------
// Static mailbox table packet
//-------------------------------------
typedef struct mailbox_table {
	ID	id;								/*!< Mailbox ID								*/
	T_CMBX	pk_cmbx;					/*!< Mailbox creation infomation packet		*/
}MBX_TBL;

//-------------------------------------
// Static mutex table packet
//-------------------------------------
typedef struct mutex_table {
	ID	id;								/*!< Mutex ID								*/
	T_CMTX	pk_cmtx;					/*!< Mutex creation infomation packet		*/
}MTX_TBL;

//-------------------------------------
// Static interrupt table packet
//-------------------------------------
typedef struct interrupt_table {
	INHNO	inhno;						/*!< Interrupt handler number to be defined	*/
	T_DINH	pk_dinh;					/*!< Pointer to the packet containing the interrupt handler definition information	*/
}INT_TBL;	

//-------------------------------------
// Static HW-ISR table packet
//-------------------------------------
typedef struct hwisr_table {
	INHNO	inhno;						/*!< Interrupt handler number to be defined	*/
	UINT	hwisr_syscall;				/*!< System call							*/
	ID		id;							/*!< Terget ID								*/
	FLGPTN	setptn;						/*!< Bit pattern (only set_flg)				*/
}HWISR_TBL;	


//=============================================================
// Syatem call parameter
//=============================================================
typedef struct {
	UINT	r4;							/*!< HW-RTOS R4				*/
    UINT    r5;                         /*!< HW-RTOS R5             */
    UINT    r6;                         /*!< HW-RTOS R6             */
    UINT    r7;                         /*!< HW-RTOS R7             */
    UINT    cmd;                        /*!< System call command    */
}SYSCALL_PARAM;

typedef struct {
	UINT    r0;                         /*!< HW-RTOS R0             */
	UINT    r1;                         /*!< HW-RTOS R1             */
}SYSCALL_RETURN;


/*==========================================================================*/
/* F U N C T I O N   P R O T O T Y P E										*/
/*==========================================================================*/
ER hwos_setup(void);
ER hwos_init(void);

/*==========================================================================*/
/* D E F I N E																*/
/*==========================================================================*/
#define MAX_CONTEXT_NUM		64					/*!< Max context number		*/
#define MAX_SEMAPHORE_NUM	128					/*!< Max semaphore number	*/
#define MAX_EVENTFLAG_NUM	64					/*!< Max eventflag number	*/
#define MAX_MAILBOX_NUM     64	                /*!< Max mailbox number     */
#define MAX_MUTEX_NUM		MAX_SEMAPHORE_NUM	/*!< Max mutex number		*/

#define MAX_PRIORITY_NUM	15					/*!< Max priority number	*/
#define MAX_SEMAPHORE_CNT	31					/*!< Max semaphore count	*/

#define TASK_TBL_END		0xffffffff			/*!< Task table end ID		*/
#define SEMAPHORE_TBL_END	0xffffffff			/*!< Semaphore table end ID	*/
#define EVENTFLAG_TBL_END	0xffffffff			/*!< Eventflag table end ID	*/
#define MAILBOX_TBL_END		0xffffffff			/*!< Mailbox table end ID	*/
#define MUTEX_TBL_END		0xffffffff			/*!< Mutex table end ID		*/
#define INT_TBL_END			0xffffffff			/*!< Interrupt table end ID	*/
#define HWISR_TBL_END		0xffffffff			/*!< HW-ISR table end ID	*/

//-------------------------------------
// HW-ISR infomation
//-------------------------------------
#define MAX_QINT_NUM       32                   /*!< Max QINT number        */

#define HWISR_SET_FLG		1					/*!< set_flg				*/
#define HWISR_SIG_SEM		2					/*!< sig_sem				*/
#define HWISR_REL_WAI		3					/*!< rel_wai				*/
#define HWISR_WUP_TSK		4					/*!< wup_tsk				*/

#endif // KERNEL_H__
